@@ -69,8 +69,13 @@ read_xml <- function(xml, pointer = TRUE, escapes = FALSE, declaration = FALSE,
       collapse = "")
 
     if (grepl(sprintf('xmlns:%s="http://schemas.openxmlformats.org/spreadsheetml/2006/main"', xml_ns), xml_file, fixed = TRUE)) {
-      xml_file <- stringi::stri_replace_all_fixed(xml_file, sprintf("<%s:", xml_ns), "<")
-      xml_file <- stringi::stri_replace_all_fixed(xml_file, sprintf("</%s:", xml_ns), "</")
+
+      xml_file <- stringi::stri_replace_all_fixed(
+        xml_file,
+        pattern = sprintf(c("xmlns:%s=", "<%s:", "</%s:"), xml_ns),
+        replacement = c("xmlns=", "<", "</"),
+        vectorize_all = FALSE
+      )
 
       # replace xml with already and cleaned output
       xml <- xml_file
