@@ -37,7 +37,7 @@ test_that("wb_remove_worksheet() is a wrapper", {
 # does not work as expected
 test_that("wb_to_df() is a wrapper", {
   wb <- wb_workbook()$add_worksheet()$add_data(x = iris)
-  expect_pseudo_wrapper("to_df")
+  expect_wrapper("to_df")
 })
 
 # wb_load() ---------------------------------------------------------------
@@ -251,7 +251,8 @@ test_that("wb_set_col_widths() is a wrapper", {
 test_that("wb_add_image() is a wrapper", {
   path <- system.file("extdata", "einstein.jpg", package = "openxlsx2")
   wb <- wb_workbook()$add_worksheet("a")
-  expect_wrapper("add_image", wb = wb, params = list(file = path, sheet = "a"))
+  # media contains a reference tmpDir
+  expect_wrapper("add_image", wb = wb, params = list(file = path, sheet = "a"), ignore_fields = c("tmpDir", "media"))
 })
 
 # wb_add_plot() -----------------------------------------------------------
@@ -273,7 +274,8 @@ test_that("wb_add_plot() is a wrapper", {
     "add_plot",
     "wb_add_plot",
     wb = wb,
-    params = list(sheet = "a")
+    params = list(sheet = "a"),
+    ignore_fields = c("tmpDir", "media")
   )
 
   # # check that it is actually working
