@@ -231,10 +231,12 @@ test_that("registering formulas works", {
 
   expect_message(wb$add_formula(x = c(YESTERDAY = fml)), "formula registered to the workbook")
   expect_error(wb$add_formula(x = c(YESTERDAY = fml)), "named regions cannot be duplicates")
-  expect_equal(wb$get_named_regions()$value, fml)
+  got <- xml_value(wb$workbook$definedNames, "definedName")
+  expect_equal(got, fml)
 
   wb <- wb_add_formula(wb, x = "YESTERDAY()", name = "YSTRDY", array = TRUE)
-  expect_equal(wb$get_named_regions()$name, c("YESTERDAY", "YSTRDY"))
+  got <- unname(unlist(xml_attr(wb$workbook$definedNames, "definedName")))
+  expect_equal(got, c("YESTERDAY", "YSTRDY"))
 
 })
 
